@@ -40,6 +40,8 @@
 #include "checksum.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &atomics_cobalt_device_vtable)
+
 #define EXITCODE(rc) (rc == LIBUSB_ERROR_TIMEOUT ? DC_STATUS_TIMEOUT : DC_STATUS_IO)
 
 #define VID 0x0471
@@ -66,7 +68,7 @@ static dc_status_t atomics_cobalt_device_set_fingerprint (dc_device_t *abstract,
 static dc_status_t atomics_cobalt_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t atomics_cobalt_device_close (dc_device_t *abstract);
 
-static const device_backend_t atomics_cobalt_device_backend = {
+static const dc_device_vtable_t atomics_cobalt_device_vtable = {
 	DC_FAMILY_ATOMICS_COBALT,
 	atomics_cobalt_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -76,15 +78,6 @@ static const device_backend_t atomics_cobalt_device_backend = {
 	atomics_cobalt_device_close /* close */
 };
 
-static int
-device_is_atomics_cobalt (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &atomics_cobalt_device_backend;
-}
-
 
 dc_status_t
 atomics_cobalt_device_open (dc_device_t **out, dc_context_t *context)
@@ -101,7 +94,7 @@ atomics_cobalt_device_open (dc_device_t **out, dc_context_t *context)
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, context, &atomics_cobalt_device_backend);
+	device_init (&device->base, context, &atomics_cobalt_device_vtable);
 
 	// Set the default values.
 	device->context = NULL;
@@ -156,9 +149,6 @@ atomics_cobalt_device_close (dc_device_t *abstract)
 {
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
-	if (! device_is_atomics_cobalt (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 #ifdef HAVE_LIBUSB
 	libusb_release_interface(device->handle, 0);
 	libusb_close (device->handle);
@@ -177,9 +167,6 @@ atomics_cobalt_device_set_fingerprint (dc_device_t *abstract, const unsigned cha
 {
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
-	if (! device_is_atomics_cobalt (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if (size && size != sizeof (device->fingerprint))
 		return DC_STATUS_INVALIDARGS;
 
@@ -197,7 +184,7 @@ atomics_cobalt_device_set_simulation (dc_device_t *abstract, unsigned int simula
 {
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
-	if (! device_is_atomics_cobalt (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	device->simulation = simulation;
@@ -211,6 +198,9 @@ atomics_cobalt_device_version (dc_device_t *abstract, unsigned char data[], unsi
 {
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
+	if (!ISINSTANCE (abstract))
+		return DC_STATUS_INVALIDARGS;
+
 	if (size < SZ_VERSION)
 		return DC_STATUS_INVALIDARGS;
 
@@ -350,9 +340,6 @@ atomics_cobalt_device_foreach (dc_device_t *abstract, dc_dive_callback_t callbac
 {
 	atomics_cobalt_device_t *device = (atomics_cobalt_device_t *) abstract;
 
-	if (! device_is_atomics_cobalt (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	progress.maximum = SZ_MEMORY + 2;
