@@ -27,6 +27,8 @@
 #include "parser-private.h"
 #include "array.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &cressi_edy_parser_vtable)
+
 typedef struct cressi_edy_parser_t cressi_edy_parser_t;
 
 struct cressi_edy_parser_t {
@@ -40,7 +42,7 @@ static dc_status_t cressi_edy_parser_get_field (dc_parser_t *abstract, dc_field_
 static dc_status_t cressi_edy_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t cressi_edy_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t cressi_edy_parser_backend = {
+static const dc_parser_vtable_t cressi_edy_parser_vtable = {
 	DC_FAMILY_CRESSI_EDY,
 	cressi_edy_parser_set_data, /* set_data */
 	cressi_edy_parser_get_datetime, /* datetime */
@@ -50,16 +52,6 @@ static const parser_backend_t cressi_edy_parser_backend = {
 };
 
 
-static int
-parser_is_cressi_edy (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &cressi_edy_parser_backend;
-}
-
-
 dc_status_t
 cressi_edy_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
@@ -74,7 +66,7 @@ cressi_edy_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &cressi_edy_parser_backend);
+	parser_init (&parser->base, context, &cressi_edy_parser_vtable);
 
 	// Set the default values.
 	parser->model = model;
@@ -88,9 +80,6 @@ cressi_edy_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 static dc_status_t
 cressi_edy_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_cressi_edy (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.
 	free (abstract);
 
@@ -101,9 +90,6 @@ cressi_edy_parser_destroy (dc_parser_t *abstract)
 static dc_status_t
 cressi_edy_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
-	if (! parser_is_cressi_edy (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	return DC_STATUS_SUCCESS;
 }
 
