@@ -30,6 +30,8 @@
 #include "checksum.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &hw_ostc_device_vtable)
+
 #define EXITCODE(rc) \
 ( \
 	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
@@ -59,7 +61,7 @@ static dc_status_t hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buff
 static dc_status_t hw_ostc_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t hw_ostc_device_close (dc_device_t *abstract);
 
-static const device_backend_t hw_ostc_device_backend = {
+static const dc_device_vtable_t hw_ostc_device_vtable = {
 	DC_FAMILY_HW_OSTC,
 	hw_ostc_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -70,16 +72,6 @@ static const device_backend_t hw_ostc_device_backend = {
 };
 
 
-static int
-device_is_hw_ostc (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &hw_ostc_device_backend;
-}
-
-
 static dc_status_t
 hw_ostc_send (hw_ostc_device_t *device, unsigned char cmd, unsigned int echo)
 {
@@ -127,7 +119,7 @@ hw_ostc_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, context, &hw_ostc_device_backend);
+	device_init (&device->base, context, &hw_ostc_device_vtable);
 
 	// Set the default values.
 	device->port = NULL;
@@ -173,9 +165,6 @@ hw_ostc_device_close (dc_device_t *abstract)
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t*) abstract;
 
-	if (! device_is_hw_ostc (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
@@ -211,9 +200,6 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t*) abstract;
 
-	if (! device_is_hw_ostc (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Erase the current contents of the buffer.
 	if (!dc_buffer_clear (buffer)) {
 		ERROR (abstract->context, "Insufficient buffer space available.");
@@ -346,7 +332,7 @@ hw_ostc_device_md2hash (dc_device_t *abstract, unsigned char data[], unsigned in
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (! device_is_hw_ostc (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < SZ_MD2HASH) {
@@ -375,7 +361,7 @@ hw_ostc_device_clock (dc_device_t *abstract, const dc_datetime_t *datetime)
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (! device_is_hw_ostc (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (datetime == NULL) {
@@ -407,7 +393,7 @@ hw_ostc_device_eeprom_read (dc_device_t *abstract, unsigned int bank, unsigned c
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (! device_is_hw_ostc (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (bank > 2) {
@@ -442,7 +428,7 @@ hw_ostc_device_eeprom_write (dc_device_t *abstract, unsigned int bank, const uns
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (! device_is_hw_ostc (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (bank > 2) {
@@ -477,7 +463,7 @@ hw_ostc_device_reset (dc_device_t *abstract)
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (! device_is_hw_ostc (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	// Send the command.
@@ -494,7 +480,7 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (! device_is_hw_ostc (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	// Erase the current contents of the buffer.
@@ -629,7 +615,7 @@ hw_ostc_extract_dives (dc_device_t *abstract, const unsigned char data[], unsign
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (abstract && !device_is_hw_ostc (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	const unsigned char header[2] = {0xFA, 0xFA};
