@@ -47,7 +47,7 @@ typedef struct mares_common_device_t {
 } mares_common_device_t;
 
 void
-mares_common_device_init (mares_common_device_t *device, dc_context_t *context, const device_backend_t *backend);
+mares_common_device_init (mares_common_device_t *device, dc_context_t *context, const dc_device_vtable_t *vtable);
 
 dc_status_t
 mares_common_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size);
