@@ -31,6 +31,8 @@
 #include "mares_common.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &mares_darwin_device_vtable)
+
 #define DARWIN    0
 #define DARWINAIR 1
 
@@ -60,7 +62,7 @@ static dc_status_t mares_darwin_device_dump (dc_device_t *abstract, dc_buffer_t
 static dc_status_t mares_darwin_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t mares_darwin_device_close (dc_device_t *abstract);
 
-static const device_backend_t mares_darwin_device_backend = {
+static const dc_device_vtable_t mares_darwin_device_vtable = {
 	DC_FAMILY_MARES_DARWIN,
 	mares_darwin_device_set_fingerprint, /* set_fingerprint */
 	mares_common_device_read, /* read */
@@ -90,14 +92,6 @@ static const mares_darwin_layout_t mares_darwinair_layout = {
 	3       /* samplesize */
 };
 
-static int
-device_is_mares_darwin (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &mares_darwin_device_backend;
-}
 
 dc_status_t
 mares_darwin_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
@@ -113,7 +107,7 @@ mares_darwin_device_open (dc_device_t **out, dc_context_t *context, const char *
 	}
 
 	// Initialize the base class.
-	mares_common_device_init (&device->base, context, &mares_darwin_device_backend);
+	mares_common_device_init (&device->base, context, &mares_darwin_device_vtable);
 
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
@@ -263,7 +257,7 @@ mares_darwin_extract_dives (dc_device_t *abstract, const unsigned char data[], u
 {
 	mares_darwin_device_t *device = (mares_darwin_device_t *) abstract;
 
-	if (!device_is_mares_darwin (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	assert (device->layout != NULL);
