@@ -27,6 +27,8 @@
 #include "parser-private.h"
 #include "array.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &mares_iconhd_parser_vtable)
+
 #define ICONHD    0x14
 #define ICONHDNET 0x15
 
@@ -43,7 +45,7 @@ static dc_status_t mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_fiel
 static dc_status_t mares_iconhd_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t mares_iconhd_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t mares_iconhd_parser_backend = {
+static const dc_parser_vtable_t mares_iconhd_parser_vtable = {
 	DC_FAMILY_MARES_ICONHD,
 	mares_iconhd_parser_set_data, /* set_data */
 	mares_iconhd_parser_get_datetime, /* datetime */
@@ -53,16 +55,6 @@ static const parser_backend_t mares_iconhd_parser_backend = {
 };
 
 
-static int
-parser_is_mares_iconhd (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &mares_iconhd_parser_backend;
-}
-
-
 dc_status_t
 mares_iconhd_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
@@ -77,7 +69,7 @@ mares_iconhd_parser_create (dc_parser_t **out, dc_context_t *context, unsigned i
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &mares_iconhd_parser_backend);
+	parser_init (&parser->base, context, &mares_iconhd_parser_vtable);
 
 	// Set the default values.
 	parser->model = model;
@@ -91,9 +83,6 @@ mares_iconhd_parser_create (dc_parser_t **out, dc_context_t *context, unsigned i
 static dc_status_t
 mares_iconhd_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_mares_iconhd (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.
 	free (abstract);
 
@@ -163,6 +152,8 @@ mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 
+	unsigned int air = (p[0] & 0x02) == 0;
+
 	if (value) {
 		switch (type) {
 		case DC_FIELD_DIVETIME:
@@ -172,11 +163,27 @@ mares_iconhd_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsi
 			*((double *) value) = array_uint16_le (p + 0x04) / 10.0;
 			break;
 		case DC_FIELD_GASMIX_COUNT:
-			*((unsigned int *) value) = 3;
+			if (air) {
+				*((unsigned int *) value) = 1;
+			} else {
+				// Count the number of active gas mixes. The active gas
+				// mixes are always first, so we stop counting as soon
+				// as the first gas marked as disabled is found.
+				unsigned int i = 0;
+				while (i < 3) {
+					if (p[0x14 + i * 4 + 1] & 0x80)
+						break;
+					i++;
+				}
+				*((unsigned int *) value) = i;
+			}
 			break;
 		case DC_FIELD_GASMIX:
+			if (air)
+				gasmix->oxygen = 0.21;
+			else
+				gasmix->oxygen = p[0x14 + flags * 4] / 100.0;
 			gasmix->helium = 0.0;
-			gasmix->oxygen = p[0x14 + flags * 4] / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
 			break;
 		default:
