@@ -32,6 +32,8 @@
 #include "ringbuffer.h"
 #include "checksum.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &oceanic_atom2_device_vtable)
+
 #define MAXRETRIES 2
 
 #define EXITCODE(rc) \
@@ -51,7 +53,7 @@ static dc_status_t oceanic_atom2_device_read (dc_device_t *abstract, unsigned in
 static dc_status_t oceanic_atom2_device_write (dc_device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size);
 static dc_status_t oceanic_atom2_device_close (dc_device_t *abstract);
 
-static const device_backend_t oceanic_atom2_device_backend = {
+static const dc_device_vtable_t oceanic_atom2_device_vtable = {
 	DC_FAMILY_OCEANIC_ATOM2,
 	oceanic_common_device_set_fingerprint, /* set_fingerprint */
 	oceanic_atom2_device_read, /* read */
@@ -87,6 +89,7 @@ static const oceanic_common_version_t oceanic_atom2b_version[] = {
 	{"ELEMENT2 \0\0 512K"},
 	{"OCEVEO20 \0\0 512K"},
 	{"TUSAZEN \0\0  512K"},
+	{"PROPLUS3 \0\0 512K"},
 };
 
 static const oceanic_common_version_t oceanic_atom2c_version[] = {
@@ -122,6 +125,10 @@ static const oceanic_common_version_t oceanic_veo1_version[] = {
 	{"OCEVEO10 \0\0   8K"},
 };
 
+static const oceanic_common_version_t oceanic_reactpro_version[] = {
+	{"REACPRO2 \0\0 512K"},
+};
+
 static const oceanic_common_layout_t aeris_f10_layout = {
 	0x10000, /* memsize */
 	0x0000, /* cf_devinfo */
@@ -239,16 +246,18 @@ static const oceanic_common_layout_t oceanic_veo1_layout = {
 	0 /* pt_mode_logbook */
 };
 
-
-static int
-device_is_oceanic_atom2 (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &oceanic_atom2_device_backend;
-}
-
+static const oceanic_common_layout_t oceanic_reactpro_layout = {
+	0xFFF0, /* memsize */
+	0x0000, /* cf_devinfo */
+	0x0040, /* cf_pointers */
+	0x0400, /* rb_logbook_begin */
+	0x0600, /* rb_logbook_end */
+	8, /* rb_logbook_entry_size */
+	0x0600, /* rb_profile_begin */
+	0xFFF0, /* rb_profile_end */
+	1, /* pt_mode_global */
+	1 /* pt_mode_logbook */
+};
 
 static dc_status_t
 oceanic_atom2_send (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char ack)
@@ -357,7 +366,7 @@ oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, const char
 	}
 
 	// Initialize the base class.
-	oceanic_common_device_init (&device->base, context, &oceanic_atom2_device_backend);
+	oceanic_common_device_init (&device->base, context, &oceanic_atom2_device_vtable);
 
 	// Set the default values.
 	device->port = NULL;
@@ -426,6 +435,8 @@ oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, const char
 		device->base.layout = &oceanic_oc1_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_veo1_version)) {
 		device->base.layout = &oceanic_veo1_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_reactpro_version)) {
+		device->base.layout = &oceanic_reactpro_layout;
 	} else {
 		device->base.layout = &oceanic_default_layout;
 	}
@@ -441,9 +452,6 @@ oceanic_atom2_device_close (dc_device_t *abstract)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
-	if (! device_is_oceanic_atom2 (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Send the quit command.
 	oceanic_atom2_quit (device);
 
@@ -465,7 +473,7 @@ oceanic_atom2_device_keepalive (dc_device_t *abstract)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
-	if (! device_is_oceanic_atom2 (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	// Send the command to the dive computer.
@@ -483,7 +491,7 @@ oceanic_atom2_device_version (dc_device_t *abstract, unsigned char data[], unsig
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
-	if (! device_is_oceanic_atom2 (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < PAGESIZE)
@@ -506,9 +514,6 @@ oceanic_atom2_device_read (dc_device_t *abstract, unsigned int address, unsigned
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
-	if (! device_is_oceanic_atom2 (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if ((address % PAGESIZE != 0) ||
 		(size    % PAGESIZE != 0))
 		return DC_STATUS_INVALIDARGS;
@@ -542,9 +547,6 @@ oceanic_atom2_device_write (dc_device_t *abstract, unsigned int address, const u
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
-	if (! device_is_oceanic_atom2 (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if ((address % PAGESIZE != 0) ||
 		(size    % PAGESIZE != 0))
 		return DC_STATUS_INVALIDARGS;
