@@ -30,6 +30,8 @@
 #include "checksum.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &reefnet_sensuspro_device_vtable)
+
 #define EXITCODE(rc) \
 ( \
 	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
@@ -52,7 +54,7 @@ static dc_status_t reefnet_sensuspro_device_dump (dc_device_t *abstract, dc_buff
 static dc_status_t reefnet_sensuspro_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t reefnet_sensuspro_device_close (dc_device_t *abstract);
 
-static const device_backend_t reefnet_sensuspro_device_backend = {
+static const dc_device_vtable_t reefnet_sensuspro_device_vtable = {
 	DC_FAMILY_REEFNET_SENSUSPRO,
 	reefnet_sensuspro_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -62,15 +64,6 @@ static const device_backend_t reefnet_sensuspro_device_backend = {
 	reefnet_sensuspro_device_close /* close */
 };
 
-static int
-device_is_reefnet_sensuspro (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &reefnet_sensuspro_device_backend;
-}
-
 
 dc_status_t
 reefnet_sensuspro_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -86,7 +79,7 @@ reefnet_sensuspro_device_open (dc_device_t **out, dc_context_t *context, const c
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, context, &reefnet_sensuspro_device_backend);
+	device_init (&device->base, context, &reefnet_sensuspro_device_vtable);
 
 	// Set the default values.
 	device->port = NULL;
@@ -134,9 +127,6 @@ reefnet_sensuspro_device_close (dc_device_t *abstract)
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
 
-	if (! device_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
@@ -155,7 +145,7 @@ reefnet_sensuspro_device_get_handshake (dc_device_t *abstract, unsigned char dat
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
 
-	if (! device_is_reefnet_sensuspro (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < SZ_HANDSHAKE) {
@@ -174,9 +164,6 @@ reefnet_sensuspro_device_set_fingerprint (dc_device_t *abstract, const unsigned
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
 
-	if (! device_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if (size && size != 4)
 		return DC_STATUS_INVALIDARGS;
 
@@ -274,9 +261,6 @@ reefnet_sensuspro_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
 
-	if (! device_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Erase the current contents of the buffer and
 	// pre-allocate the required amount of memory.
 	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, SZ_MEMORY)) {
@@ -330,9 +314,6 @@ reefnet_sensuspro_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 static dc_status_t
 reefnet_sensuspro_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
-	if (! device_is_reefnet_sensuspro (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	dc_buffer_t *buffer = dc_buffer_new (SZ_MEMORY);
 	if (buffer == NULL)
 		return DC_STATUS_NOMEMORY;
@@ -357,7 +338,7 @@ reefnet_sensuspro_device_write_interval (dc_device_t *abstract, unsigned char in
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
 
-	if (! device_is_reefnet_sensuspro (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (interval < 1 || interval > 127)
@@ -385,7 +366,7 @@ reefnet_sensuspro_extract_dives (dc_device_t *abstract, const unsigned char data
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
 
-	if (abstract && !device_is_reefnet_sensuspro (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	const unsigned char header[4] = {0x00, 0x00, 0x00, 0x00};
