@@ -30,6 +30,8 @@
 #include "serial.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &suunto_solution_device_vtable)
+
 #define EXITCODE(rc) \
 ( \
 	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
@@ -49,7 +51,7 @@ static dc_status_t suunto_solution_device_dump (dc_device_t *abstract, dc_buffer
 static dc_status_t suunto_solution_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t suunto_solution_device_close (dc_device_t *abstract);
 
-static const device_backend_t suunto_solution_device_backend = {
+static const dc_device_vtable_t suunto_solution_device_vtable = {
 	DC_FAMILY_SUUNTO_SOLUTION,
 	NULL, /* set_fingerprint */
 	NULL, /* read */
@@ -59,15 +61,6 @@ static const device_backend_t suunto_solution_device_backend = {
 	suunto_solution_device_close /* close */
 };
 
-static int
-device_is_suunto_solution (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &suunto_solution_device_backend;
-}
-
 
 dc_status_t
 suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -83,7 +76,7 @@ suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const cha
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, context, &suunto_solution_device_backend);
+	device_init (&device->base, context, &suunto_solution_device_vtable);
 
 	// Set the default values.
 	device->port = NULL;
@@ -132,9 +125,6 @@ suunto_solution_device_close (dc_device_t *abstract)
 {
 	suunto_solution_device_t *device = (suunto_solution_device_t*) abstract;
 
-	if (! device_is_suunto_solution (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
@@ -153,9 +143,6 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	suunto_solution_device_t *device = (suunto_solution_device_t*) abstract;
 
-	if (! device_is_suunto_solution (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Erase the current contents of the buffer and
 	// allocate the required amount of memory.
 	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, SZ_MEMORY)) {
@@ -261,9 +248,6 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 static dc_status_t
 suunto_solution_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
-	if (! device_is_suunto_solution (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	dc_buffer_t *buffer = dc_buffer_new (SZ_MEMORY);
 	if (buffer == NULL)
 		return DC_STATUS_NOMEMORY;
@@ -298,7 +282,7 @@ suunto_solution_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 dc_status_t
 suunto_solution_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
-	if (abstract && !device_is_suunto_solution (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < SZ_MEMORY)
