@@ -27,6 +27,8 @@
 #include "context-private.h"
 #include "parser-private.h"
 
+#define ISINSTANCE(parser) dc_parser_isinstance((parser), &suunto_solution_parser_vtable)
+
 typedef struct suunto_solution_parser_t suunto_solution_parser_t;
 
 struct suunto_solution_parser_t {
@@ -42,7 +44,7 @@ static dc_status_t suunto_solution_parser_get_field (dc_parser_t *abstract, dc_f
 static dc_status_t suunto_solution_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
 static dc_status_t suunto_solution_parser_destroy (dc_parser_t *abstract);
 
-static const parser_backend_t suunto_solution_parser_backend = {
+static const dc_parser_vtable_t suunto_solution_parser_vtable = {
 	DC_FAMILY_SUUNTO_SOLUTION,
 	suunto_solution_parser_set_data, /* set_data */
 	NULL, /* datetime */
@@ -52,16 +54,6 @@ static const parser_backend_t suunto_solution_parser_backend = {
 };
 
 
-static int
-parser_is_suunto_solution (dc_parser_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &suunto_solution_parser_backend;
-}
-
-
 dc_status_t
 suunto_solution_parser_create (dc_parser_t **out, dc_context_t *context)
 {
@@ -76,7 +68,7 @@ suunto_solution_parser_create (dc_parser_t **out, dc_context_t *context)
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, context, &suunto_solution_parser_backend);
+	parser_init (&parser->base, context, &suunto_solution_parser_vtable);
 
 	// Set the default values.
 	parser->cached = 0;
@@ -92,9 +84,6 @@ suunto_solution_parser_create (dc_parser_t **out, dc_context_t *context)
 static dc_status_t
 suunto_solution_parser_destroy (dc_parser_t *abstract)
 {
-	if (! parser_is_suunto_solution (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Free memory.
 	free (abstract);
 
@@ -107,9 +96,6 @@ suunto_solution_parser_set_data (dc_parser_t *abstract, const unsigned char *dat
 {
 	suunto_solution_parser_t *parser = (suunto_solution_parser_t *) abstract;
 
-	if (! parser_is_suunto_solution (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
@@ -188,9 +174,6 @@ suunto_solution_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, u
 static dc_status_t
 suunto_solution_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata)
 {
-	if (! parser_is_suunto_solution (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
