@@ -32,6 +32,8 @@
 #include "checksum.h"
 #include "array.h"
 
+#define ISINSTANCE(device) dc_device_isinstance((device), &suunto_vyper_device_vtable)
+
 #define EXITCODE(rc) \
 ( \
 	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
@@ -59,7 +61,7 @@ static dc_status_t suunto_vyper_device_dump (dc_device_t *abstract, dc_buffer_t
 static dc_status_t suunto_vyper_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
 static dc_status_t suunto_vyper_device_close (dc_device_t *abstract);
 
-static const device_backend_t suunto_vyper_device_backend = {
+static const dc_device_vtable_t suunto_vyper_device_vtable = {
 	DC_FAMILY_SUUNTO_VYPER,
 	suunto_common_device_set_fingerprint, /* set_fingerprint */
 	suunto_vyper_device_read, /* read */
@@ -86,16 +88,6 @@ static const suunto_common_layout_t suunto_spyder_layout = {
 };
 
 
-static int
-device_is_suunto_vyper (dc_device_t *abstract)
-{
-	if (abstract == NULL)
-		return 0;
-
-    return abstract->backend == &suunto_vyper_device_backend;
-}
-
-
 dc_status_t
 suunto_vyper_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
@@ -110,7 +102,7 @@ suunto_vyper_device_open (dc_device_t **out, dc_context_t *context, const char *
 	}
 
 	// Initialize the base class.
-	suunto_common_device_init (&device->base, context, &suunto_vyper_device_backend);
+	suunto_common_device_init (&device->base, context, &suunto_vyper_device_vtable);
 
 	// Set the default values.
 	device->port = NULL;
@@ -165,9 +157,6 @@ suunto_vyper_device_close (dc_device_t *abstract)
 {
 	suunto_vyper_device_t *device = (suunto_vyper_device_t*) abstract;
 
-	if (! device_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
@@ -266,9 +255,6 @@ suunto_vyper_device_read (dc_device_t *abstract, unsigned int address, unsigned
 {
 	suunto_vyper_device_t *device = (suunto_vyper_device_t*) abstract;
 
-	if (! device_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
 		// Calculate the package size.
@@ -302,9 +288,6 @@ suunto_vyper_device_write (dc_device_t *abstract, unsigned int address, const un
 {
 	suunto_vyper_device_t *device = (suunto_vyper_device_t*) abstract;
 
-	if (! device_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
 		// Calculate the package size.
@@ -460,7 +443,7 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 dc_status_t
 suunto_vyper_device_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init)
 {
-	if (! device_is_suunto_vyper (abstract))
+	if (!ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	return suunto_vyper_read_dive (abstract, buffer, init, NULL);
@@ -470,9 +453,6 @@ suunto_vyper_device_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int i
 static dc_status_t
 suunto_vyper_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
-	if (! device_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Erase the current contents of the buffer and
 	// allocate the required amount of memory.
 	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, SZ_MEMORY)) {
@@ -490,9 +470,6 @@ suunto_vyper_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback,
 {
 	suunto_common_device_t *device = (suunto_common_device_t*) abstract;
 
-	if (! device_is_suunto_vyper (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	progress.maximum = SZ_MEMORY;
@@ -581,7 +558,7 @@ suunto_vyper_extract_dives (dc_device_t *abstract, const unsigned char data[], u
 {
 	suunto_common_device_t *device = (suunto_common_device_t*) abstract;
 
-	if (abstract && !device_is_suunto_vyper (abstract))
+	if (abstract && !ISINSTANCE (abstract))
 		return DC_STATUS_INVALIDARGS;
 
 	if (size < SZ_MEMORY)
